@@ -8,7 +8,7 @@ from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.web.util import formatFailure
 
-from buildbot import interfaces
+from buildbot import interfaces, locks
 from buildbot.status import progress
 from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, SKIPPED, \
      EXCEPTION
@@ -433,7 +433,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
 
     def __init__(self, workdir, command, env=None, 
                  want_stdout=1, want_stderr=1,
-                 timeout=20*60, logfiles={}, **kwargs):
+                 timeout=20*60, logfiles={}, usePTY="slave-config"):
         """
         @type  workdir: string
         @param workdir: directory where the command ought to run,
@@ -486,6 +486,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
                 'want_stderr': want_stderr,
                 'logfiles': logfiles,
                 'timeout': timeout,
+                'usePTY': usePTY,
                 }
         LoggedRemoteCommand.__init__(self, "shell", args)
 
@@ -502,7 +503,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
         return LoggedRemoteCommand.start(self)
 
     def __repr__(self):
-        return "<RemoteShellCommand '%s'>" % self.command
+        return "<RemoteShellCommand '%s'>" % repr(self.command)
 
 class BuildStep:
     """
@@ -544,11 +545,14 @@ class BuildStep:
     # immediately, the others will be taken into consideration when
     # determining the overall build status.
     #
+    # steps that are makred as alwaysRun will be run regardless of the outcome
+    # of previous steps (especially steps with haltOnFailure=True)
     haltOnFailure = False
     flunkOnWarnings = False
     flunkOnFailure = False
     warnOnWarnings = False
     warnOnFailure = False
+    alwaysRun = False
 
     # 'parms' holds a list of all the parameters we care about, to allow
     # users to instantiate a subclass of BuildStep with a mixture of
@@ -563,6 +567,7 @@ class BuildStep:
              'flunkOnFailure',
              'warnOnWarnings',
              'warnOnFailure',
+             'alwaysRun',
              'progressMetrics',
              ]
 
@@ -598,11 +603,11 @@ class BuildStep:
         self.buildslave = buildslave
 
     def setDefaultWorkdir(self, workdir):
-        # the Build calls this just after __init__ and setDefaultWorkdir.
-        # ShellCommand and variants use a slave-side workdir, but some other
-        # steps do not. Subclasses which use a workdir should use the value
-        # set by this method unless they were constructed with something more
-        # specific.
+        # The Build calls this just after __init__().  ShellCommand
+        # and variants use a slave-side workdir, but some other steps
+        # do not. Subclasses which use a workdir should use the value
+        # set by this method unless they were constructed with
+        # something more specific.
         pass
 
     def addFactoryArguments(self, **kwargs):
@@ -662,12 +667,18 @@ class BuildStep:
         self.remote = remote
         self.deferred = defer.Deferred()
         # convert all locks into their real form
-        self.locks = [self.build.builder.botmaster.getLockByID(l)
-                      for l in self.locks]
+        lock_list = []
+        for access in self.locks:
+            if not isinstance(access, locks.LockAccess):
+                # Buildbot 0.7.7 compability: user did not specify access
+                access = access.defaultAccess()
+            lock = self.build.builder.botmaster.getLockByID(access.lockid)
+            lock_list.append((lock, access))
+        self.locks = lock_list
         # then narrow SlaveLocks down to the slave that this build is being
         # run on
-        self.locks = [l.getLock(self.build.slavebuilder) for l in self.locks]
-        for l in self.locks:
+        self.locks = [(l.getLock(self.build.slavebuilder), la) for l, la in self.locks]
+        for l, la in self.locks:
             if l in self.build.locks:
                 log.msg("Hey, lock %s is claimed by both a Step (%s) and the"
                         " parent Build (%s)" % (l, self, self.build))
@@ -680,15 +691,15 @@ class BuildStep:
         log.msg("acquireLocks(step %s, locks %s)" % (self, self.locks))
         if not self.locks:
             return defer.succeed(None)
-        for lock in self.locks:
-            if not lock.isAvailable():
+        for lock, access in self.locks:
+            if not lock.isAvailable(access):
                 log.msg("step %s waiting for lock %s" % (self, lock))
-                d = lock.waitUntilMaybeAvailable(self)
+                d = lock.waitUntilMaybeAvailable(self, access)
                 d.addCallback(self.acquireLocks)
                 return d
         # all locks are available, claim them all
-        for lock in self.locks:
-            lock.claim(self)
+        for lock, access in self.locks:
+            lock.claim(self, access)
         return defer.succeed(None)
 
     def _startStep_2(self, res):
@@ -698,8 +709,9 @@ class BuildStep:
         try:
             skip = self.start()
             if skip == SKIPPED:
-                reactor.callLater(0, self.releaseLocks)
-                reactor.callLater(0, self.deferred.callback, SKIPPED)
+                # this return value from self.start is a shortcut
+                # to finishing the step immediately
+                reactor.callLater(0, self.finished, SKIPPED)
         except:
             log.msg("BuildStep.startStep exception in .start")
             self.failed(Failure())
@@ -718,7 +730,6 @@ class BuildStep:
         As the step runs, it should send status information to the
         BuildStepStatus::
 
-          self.step_status.setColor('red')
           self.step_status.setText(['compile', 'failed'])
           self.step_status.setText2(['4', 'warnings'])
 
@@ -769,8 +780,8 @@ class BuildStep:
 
     def releaseLocks(self):
         log.msg("releaseLocks(%s): %s" % (self, self.locks))
-        for lock in self.locks:
-            lock.release(self)
+        for lock, access in self.locks:
+            lock.release(self, access)
 
     def finished(self, results):
         if self.progress:
@@ -791,7 +802,6 @@ class BuildStep:
             self.addHTMLLog("err.html", formatFailure(why))
             self.addCompleteLog("err.text", why.getTraceback())
             # could use why.getDetailedTraceback() for more information
-            self.step_status.setColor("purple")
             self.step_status.setText([self.name, "exception"])
             self.step_status.setText2([self.name])
             self.step_status.stepFinished(EXCEPTION)
@@ -943,9 +953,9 @@ class LoggingBuildStep(BuildStep):
         @param cmd: a suitable RemoteCommand which will be launched, with
                     all output being put into our self.stdio_log LogFile
         """
-        log.msg("ShellCommand.startCommand(cmd=%s)", (cmd,))
+        log.msg("ShellCommand.startCommand(cmd=%s)" % (cmd,))
+        log.msg("  cmd.args = %r" % (cmd.args))
         self.cmd = cmd # so we can interrupt it
-        self.step_status.setColor("yellow")
         self.step_status.setText(self.describe(False))
 
         # stdio is the first log
@@ -990,7 +1000,6 @@ class LoggingBuildStep(BuildStep):
 
     def checkDisconnect(self, f):
         f.trap(error.ConnectionLost)
-        self.step_status.setColor("red")
         self.step_status.setText(self.describe(True) +
                                  ["failed", "slave", "lost"])
         self.step_status.setText2(["failed", "slave", "lost"])
@@ -1075,19 +1084,9 @@ class LoggingBuildStep(BuildStep):
                 return self.getText2(cmd, results)
         return []
 
-    def getColor(self, cmd, results):
-        assert results in (SUCCESS, WARNINGS, FAILURE)
-        if results == SUCCESS:
-            return "green"
-        elif results == WARNINGS:
-            return "orange"
-        else:
-            return "red"
-
     def setStatus(self, cmd, results):
         # this is good enough for most steps, but it can be overridden to
         # get more control over the displayed text
-        self.step_status.setColor(self.getColor(cmd, results))
         self.step_status.setText(self.getText(cmd, results))
         self.step_status.setText2(self.maybeGetText2(cmd, results))
 
