@@ -13,6 +13,8 @@ from buildbot.interfaces import IControl, IStatusReceiver
 from buildbot.status.web.base import HtmlResource, Box, \
      build_get_class, ICurrentBox, OneLineMixin, map_branches, \
      make_stop_form, make_force_build_form
+from buildbot.status.web.feeds import Rss20StatusResource, \
+     Atom10StatusResource
 from buildbot.status.web.waterfall import WaterfallStatusResource
 from buildbot.status.web.grid import GridStatusResource
 from buildbot.status.web.changes import ChangesResource
@@ -217,7 +219,7 @@ class OneBoxPerBuilder(HtmlResource):
                     label = "#%d" % b.getNumber()
                 text = ['<a href="%s">%s</a>' % (url, label)]
                 text.extend(b.getText())
-                box = Box(text, b.getColor(),
+                box = Box(text,
                           class_="LastBuild box %s" % build_get_class(b))
                 data += box.td(align="center")
             else:
@@ -285,6 +287,12 @@ class WebStatus(service.MultiService):
                   to individual changes, builders, builds, steps, and logs.
                   A number of query-arguments can be added to influence
                   the display.
+     /rss : a rss feed summarizing all failed builds. The same
+            query-arguments used by 'waterfall' can be added to
+            influence the feed output.
+     /atom : an atom feed summarizing all failed builds. The same
+             query-arguments used by 'waterfall' can be added to
+             influence the feed output.
      /grid : another summary display that shows a grid of builds, with
              sourcestamps on the x axis, and builders on the y.  Query 
              arguments similar to those for the waterfall can be added.
@@ -360,7 +368,7 @@ class WebStatus(service.MultiService):
     # all the changes).
 
     def __init__(self, http_port=None, distrib_port=None, allowForce=False,
-                       public_html="public_html"):
+                       public_html="public_html", site=None):
         """Run a web server that provides Buildbot status.
 
         @type  http_port: int or L{twisted.application.strports} string
@@ -400,6 +408,10 @@ class WebStatus(service.MultiService):
         @param public_html: the path to the public_html directory for this display,
                             either absolute or relative to the basedir.  The default
                             is 'public_html', which selects BASEDIR/public_html.
+
+        @type site: None or L{twisted.web.server.Site}
+        @param site: Use this if you want to define your own object instead of
+                     using the default.`
         """
 
         service.MultiService.__init__(self)
@@ -415,10 +427,14 @@ class WebStatus(service.MultiService):
         self.allowForce = allowForce
         self.public_html = public_html
 
-        # this will be replaced once we've been attached to a parent (and
-        # thus have a basedir and can reference BASEDIR)
-        root = static.Data("placeholder", "text/plain")
-        self.site = server.Site(root)
+        # If we were given a site object, go ahead and use it.
+        if site:
+            self.site = site
+        else:
+            # this will be replaced once we've been attached to a parent (and
+            # thus have a basedir and can reference BASEDIR)
+            root = static.Data("placeholder", "text/plain")
+            self.site = server.Site(root)
         self.childrenToBeAdded = {}
 
         self.setupUsualPages()
@@ -469,12 +485,19 @@ class WebStatus(service.MultiService):
 
     def setServiceParent(self, parent):
         service.MultiService.setServiceParent(self, parent)
+
+        # this class keeps a *separate* link to the buildmaster, rather than
+        # just using self.parent, so that when we are "disowned" (and thus
+        # parent=None), any remaining HTTP clients of this WebStatus will still
+        # be able to get reasonable results.
+        self.master = parent
+
         self.setupSite()
 
     def setupSite(self):
         # this is responsible for creating the root resource. It isn't done
         # at __init__ time because we need to reference the parent's basedir.
-        htmldir = os.path.abspath(os.path.join(self.parent.basedir, self.public_html))
+        htmldir = os.path.abspath(os.path.join(self.master.basedir, self.public_html))
         if os.path.isdir(htmldir):
             log.msg("WebStatus using (%s)" % htmldir)
         else:
@@ -489,6 +512,10 @@ class WebStatus(service.MultiService):
         for name, child_resource in self.childrenToBeAdded.iteritems():
             root.putChild(name, child_resource)
 
+        status = self.getStatus()
+        root.putChild("rss", Rss20StatusResource(status))
+        root.putChild("atom", Atom10StatusResource(status))
+
         self.site.resource = root
 
     def putChild(self, name, child_resource):
@@ -509,12 +536,15 @@ class WebStatus(service.MultiService):
         return service.MultiService.stopService(self)
 
     def getStatus(self):
-        return self.parent.getStatus()
+        return self.master.getStatus()
+
     def getControl(self):
         if self.allowForce:
-            return IControl(self.parent)
+            return IControl(self.master)
         return None
 
+    def getChangeSvc(self):
+        return self.master.change_svc
     def getPortnum(self):
         # this is for the benefit of unit tests
         s = list(self)[0]
