@@ -6,6 +6,7 @@ from twisted.web import html
 import urllib
 
 import time
+import operator
 
 from buildbot import interfaces, util
 from buildbot import version
@@ -23,9 +24,9 @@ class CurrentBox(components.Adapter):
     def formatETA(self, prefix, eta):
         if eta is None:
             return []
-        if eta < 0:
-            return ["Soon"]
-        eta_parts = []
+        if eta < 60:
+            return ["< 1 min"]
+        eta_parts = ["~"]
         eta_secs = eta
         if eta_secs > 3600:
             eta_parts.append("%d hrs" % (eta_secs / 3600))
@@ -33,9 +34,8 @@ class CurrentBox(components.Adapter):
         if eta_secs > 60:
             eta_parts.append("%d mins" % (eta_secs / 60))
             eta_secs %= 60
-        eta_parts.append("%d secs" % eta_secs)
-        abstime = time.strftime("%H:%M:%S", time.localtime(util.now()+eta))
-        return [prefix, ", ".join(eta_parts), "at %s" % abstime]
+        abstime = time.strftime("%H:%M", time.localtime(util.now()+eta))
+        return [prefix, " ".join(eta_parts), "at %s" % abstime]
 
     def getBox(self, status):
         # getState() returns offline, idle, or building
@@ -55,24 +55,19 @@ class CurrentBox(components.Adapter):
             state = "waiting"
 
         if state == "building":
-            color = "yellow"
             text = ["building"]
             if builds:
                 for b in builds:
                     eta = b.getETA()
                     text.extend(self.formatETA("ETA in", eta))
         elif state == "offline":
-            color = "red"
             text = ["offline"]
         elif state == "idle":
-            color = "white"
             text = ["idle"]
         elif state == "waiting":
-            color = "yellow"
             text = ["waiting"]
         else:
             # just in case I add a state and forget to update this
-            color = "white"
             text = [state]
 
         # TODO: for now, this pending/upcoming stuff is in the "current
@@ -87,7 +82,7 @@ class CurrentBox(components.Adapter):
         for t in upcoming:
             eta = t - util.now()
             text.extend(self.formatETA("next in", eta))
-        return Box(text, color=color, class_="Activity " + state)
+        return Box(text, class_="Activity " + state)
 
 components.registerAdapter(CurrentBox, builder.BuilderStatus, ICurrentBox)
 
@@ -104,17 +99,18 @@ class BuildTopBox(components.Adapter):
         builds = list(builder.generateFinishedBuilds(map_branches(branches),
                                                      num_builds=1))
         if not builds:
-            return Box(["none"], "white", class_="LastBuild")
+            return Box(["none"], class_="LastBuild")
         b = builds[0]
         name = b.getBuilder().getName()
         number = b.getNumber()
         url = path_to_build(req, b)
         text = b.getText()
+        tests_failed = b.getSummaryStatistic('tests-failed', operator.add, 0)
+        if tests_failed: text.extend(["Failed tests: %d" % tests_failed])
         # TODO: maybe add logs?
         # TODO: add link to the per-build page at 'url'
-        c = b.getColor()
         class_ = build_get_class(b)
-        return Box(text, c, class_="LastBuild %s" % class_)
+        return Box(text, class_="LastBuild %s" % class_)
 components.registerAdapter(BuildTopBox, builder.BuilderStatus, ITopBox)
 
 class BuildBox(components.Adapter):
@@ -128,15 +124,12 @@ class BuildBox(components.Adapter):
         reason = b.getReason()
         text = ('<a title="Reason: %s" href="%s">Build %d</a>'
                 % (html.escape(reason), url, number))
-        color = "yellow"
         class_ = "start"
         if b.isFinished() and not b.getSteps():
             # the steps have been pruned, so there won't be any indication
-            # of whether it succeeded or failed. Color the box red or green
-            # to show its status
-            color = b.getColor()
+            # of whether it succeeded or failed.
             class_ = build_get_class(b)
-        return Box([text], color=color, class_="BuildStep " + class_)
+        return Box([text], class_="BuildStep " + class_)
 components.registerAdapter(BuildBox, builder.BuildStatus, IBox)
 
 class StepBox(components.Adapter):
@@ -162,9 +155,8 @@ class StepBox(components.Adapter):
         for name, target in urls.items():
             text.append('[<a href="%s" class="%s">%s</a>]' %
                         (target, ex_url_class, html.escape(name)))
-        color = self.original.getColor()
         class_ = "BuildStep " + build_get_class(self.original)
-        return Box(text, color, class_=class_)
+        return Box(text, class_=class_)
 components.registerAdapter(StepBox, builder.BuildStepStatus, IBox)
 
 
@@ -173,11 +165,8 @@ class EventBox(components.Adapter):
 
     def getBox(self, req):
         text = self.original.getText()
-        color = self.original.getColor()
         class_ = "Event"
-        if color:
-            class_ += " " + color
-        return Box(text, color, class_=class_)
+        return Box(text, class_=class_)
 components.registerAdapter(EventBox, builder.Event, IBox)
         
 
@@ -192,8 +181,6 @@ class Spacer:
         return (self.started, self.finished)
     def getText(self):
         return []
-    def getColor(self):
-        return None
 
 class SpacerBox(components.Adapter):
     implements(IBox)
@@ -423,7 +410,7 @@ class WaterfallStatusResource(HtmlResource):
 
     def getChangemaster(self, request):
         # TODO: this wants to go away, access it through IStatus
-        return request.site.buildbot_service.parent.change_svc
+        return request.site.buildbot_service.getChangeSvc()
 
     def get_reload_time(self, request):
         if "reload" in request.args:
@@ -528,7 +515,7 @@ class WaterfallStatusResource(HtmlResource):
 
         data += "</table>\n"
 
-        data += "<hr />\n"
+        data += '<hr /><div class="footer">\n'
 
         def with_args(req, remove_args=[], new_args=[], new_path=None):
             # sigh, nevow makes this sort of manipulation easier
@@ -591,6 +578,7 @@ class WaterfallStatusResource(HtmlResource):
                  time.strftime("%a %d %b %Y %H:%M:%S",
                                time.localtime(util.now()))
                  + "\n")
+        data += '</div>\n'
         return data
 
     def body0(self, request, builders):
@@ -608,14 +596,12 @@ class WaterfallStatusResource(HtmlResource):
         data += td("", colspan=2)
         for b in builders:
             text = ""
-            color = "#ca88f7"
             state, builds = b.getState()
             if state != "offline":
                 text += "%s<br />\n" % state #b.getCurrentBig().text[0]
             else:
                 text += "OFFLINE<br />\n"
-                color = "#ffe0e0"
-            data += td(text, align="center", bgcolor=color)
+            data += td(text, align="center")
 
         # the next row has the column headers: time, changes, builder names
         data += " <tr>\n"
@@ -632,7 +618,7 @@ class WaterfallStatusResource(HtmlResource):
         data += td("04:00", align="bottom")
         data += td("fred", align="center")
         for name in names:
-            data += td("stuff", align="center", bgcolor="red")
+            data += td("stuff", align="center")
         data += " </tr>\n"
 
         data += "</table>\n"
@@ -790,9 +776,8 @@ class WaterfallStatusResource(HtmlResource):
                     for e in row[c]:
                         log.msg("Event", r, c, sourceNames[c], e.getText())
                         lognames = [loog.getName() for loog in e.getLogs()]
-                        data += "%s: %s: %s %s<br />" % (e.getText(),
+                        data += "%s: %s: %s<br />" % (e.getText(),
                                                          e.getTimes()[0],
-                                                         e.getColor(),
                                                          lognames)
                 else:
                     data += "<b>%s</b> [none]<br />\n" % sourceNames[c]
