@@ -56,6 +56,7 @@ class ShellCommand(LoggingBuildStep):
     def __init__(self, workdir=None,
                  description=None, descriptionDone=None,
                  command=None,
+                 usePTY="slave-config",
                  **kwargs):
         # most of our arguments get passed through to the RemoteShellCommand
         # that we create, but first strip out the ones that we pass to
@@ -71,7 +72,7 @@ class ShellCommand(LoggingBuildStep):
         if isinstance(self.descriptionDone, str):
             self.descriptionDone = [self.descriptionDone]
         if command:
-            self.command = command
+            self.setCommand(command)
 
         # pull out the ones that LoggingBuildStep wants, then upcall
         buildstep_kwargs = {}
@@ -87,6 +88,7 @@ class ShellCommand(LoggingBuildStep):
 
         # everything left over goes to the RemoteShellCommand
         kwargs['workdir'] = workdir # including a copy of 'workdir'
+        kwargs['usePTY'] = usePTY
         self.remote_kwargs = kwargs
         # we need to stash the RemoteShellCommand's args too
         self.addFactoryArguments(**kwargs)
@@ -116,9 +118,9 @@ class ShellCommand(LoggingBuildStep):
         """
 
         if done and self.descriptionDone is not None:
-            return self.descriptionDone
+            return list(self.descriptionDone)
         if self.description is not None:
-            return self.description
+            return list(self.description)
 
         properties = self.build.getProperties()
         words = self.command
@@ -135,17 +137,20 @@ class ShellCommand(LoggingBuildStep):
         return ["'%s" % words[0], "%s" % words[1], "...'"]
 
     def setupEnvironment(self, cmd):
-        # XXX is this used? documented? replaced by properties?
-        # merge in anything from Build.slaveEnvironment . Earlier steps
-        # (perhaps ones which compile libraries or sub-projects that need to
-        # be referenced by later steps) can add keys to
-        # self.build.slaveEnvironment to affect later steps.
+        # merge in anything from Build.slaveEnvironment
+        # This can be set from a Builder-level environment, or from earlier
+        # BuildSteps. The latter method is deprecated and superceded by
+        # BuildProperties.
+        # Environment variables passed in by a BuildStep override
+        # those passed in at the Builder level.
         properties = self.build.getProperties()
         slaveEnv = self.build.slaveEnvironment
         if slaveEnv:
             if cmd.args['env'] is None:
                 cmd.args['env'] = {}
-            cmd.args['env'].update(properties.render(slaveEnv))
+            fullSlaveEnv = slaveEnv.copy()
+            fullSlaveEnv.update(cmd.args['env'])
+            cmd.args['env'] = properties.render(fullSlaveEnv)
             # note that each RemoteShellCommand gets its own copy of the
             # dictionary, so we shouldn't be affecting anyone but ourselves.
 
@@ -177,15 +182,24 @@ class ShellCommand(LoggingBuildStep):
         # the Source subclasses) can just skip straight to startCommand()
         properties = self.build.getProperties()
 
+        warnings = []
+
         # create the actual RemoteShellCommand instance now
         kwargs = properties.render(self.remote_kwargs)
         kwargs['command'] = properties.render(self.command)
         kwargs['logfiles'] = self.logfiles
+
+        # check for the usePTY flag
+        if kwargs.has_key('usePTY') and kwargs['usePTY'] != 'slave-config':
+            slavever = self.slaveVersion("shell", "old")
+            if self.slaveVersionIsOlderThan("svn", "2.7"):
+                warnings.append("NOTE: slave does not allow master to override usePTY\n")
+
         cmd = RemoteShellCommand(**kwargs)
         self.setupEnvironment(cmd)
         self.checkForOldSlaveAndLogfiles()
 
-        self.startCommand(cmd)
+        self.startCommand(cmd, warnings)
 
 
 
@@ -272,6 +286,7 @@ class Configure(ShellCommand):
 
     name = "configure"
     haltOnFailure = 1
+    flunkOnFailure = 1
     description = ["configuring"]
     descriptionDone = ["configure"]
     command = ["./configure"]
@@ -280,21 +295,17 @@ class WarningCountingShellCommand(ShellCommand):
     warnCount = 0
     warningPattern = '.*warning[: ].*'
 
-    def __init__(self, **kwargs):
+    def __init__(self, warningPattern=None, **kwargs):
         # See if we've been given a regular expression to use to match
         # warnings. If not, use a default that assumes any line with "warning"
         # present is a warning. This may lead to false positives in some cases.
-        wp = None
-        if kwargs.has_key('warningPattern'):
-            wp = kwargs['warningPattern']
-            del kwargs['warningPattern']
-            self.warningPattern = wp
+        if warningPattern:
+            self.warningPattern = warningPattern
 
         # And upcall to let the base class do its work
         ShellCommand.__init__(self, **kwargs)
 
-        if wp:
-            self.addFactoryArguments(warningPattern=wp)
+        self.addFactoryArguments(warningPattern=warningPattern)
 
     def createSummary(self, log):
         self.warnCount = 0
@@ -346,6 +357,7 @@ class Compile(WarningCountingShellCommand):
 
     name = "compile"
     haltOnFailure = 1
+    flunkOnFailure = 1
     description = ["compiling"]
     descriptionDone = ["compile"]
     command = ["make", "all"]
@@ -354,11 +366,10 @@ class Compile(WarningCountingShellCommand):
     # things to track: number of files compiled, number of directories
     # traversed (assuming 'make' is being used)
 
-    def createSummary(self, cmd):
+    def createSummary(self, log):
         # TODO: grep for the characteristic GCC error lines and
         # assemble them into a pair of buffers
-        WarningCountingShellCommand.createSummary(self, cmd)
-        pass
+        WarningCountingShellCommand.createSummary(self, log)
 
 class Test(WarningCountingShellCommand):
 
@@ -382,63 +393,95 @@ class Test(WarningCountingShellCommand):
         passed += self.step_status.getStatistic('tests-passed', 0)
         self.step_status.setStatistic('tests-passed', passed)
 
-    def getText(self, cmd, results):
-        text = WarningCountingShellCommand.getText(self, cmd, results)
-        if self.step_status.hasStatistic('tests-total'):
-            total = self.step_status.getStatistic("tests-total", 0)
-            failed = self.step_status.getStatistic("tests-failed", 0)
-            passed = self.step_status.getStatistic("tests-passed", 0)
-            warnings = self.step_status.getStatistic("tests-warnings", 0)
-            if not total:
-                total = failed + passed + warnings
+    def describe(self, done=False):
+        description = WarningCountingShellCommand.describe(self, done)
+        if done:
+            if self.step_status.hasStatistic('tests-total'):
+                total = self.step_status.getStatistic("tests-total", 0)
+                failed = self.step_status.getStatistic("tests-failed", 0)
+                passed = self.step_status.getStatistic("tests-passed", 0)
+                warnings = self.step_status.getStatistic("tests-warnings", 0)
+                if not total:
+                    total = failed + passed + warnings
 
-            if total:
-                text.append('%d tests' % total)
-            if passed:
-                text.append('%d passed' % passed)
-            if warnings:
-                text.append('%d warnings' % warnings)
-            if failed:
-                text.append('%d failed' % failed)
-        return text
+                if total:
+                    description.append('%d tests' % total)
+                if passed:
+                    description.append('%d passed' % passed)
+                if warnings:
+                    description.append('%d warnings' % warnings)
+                if failed:
+                    description.append('%d failed' % failed)
+        return description
 
 class PerlModuleTest(Test):
     command=["prove", "--lib", "lib", "-r", "t"]
     total = 0
 
     def evaluateCommand(self, cmd):
-        lines = self.getLog('stdio').readlines()
+        # Get stdio, stripping pesky newlines etc.
+        lines = map(
+            lambda line : line.replace('\r\n','').replace('\r','').replace('\n',''),
+            self.getLog('stdio').readlines()
+            )
 
-        re_test_result = re.compile("^(All tests successful)|(\d+)/(\d+) subtests failed|Files=\d+, Tests=(\d+),")
+        total = 0
+        passed = 0
+        failed = 0
+        rc = cmd.rc
 
-        mos = map(lambda line: re_test_result.search(line), lines)
-        test_result_lines = [mo.groups() for mo in mos if mo]
+        # New version of Test::Harness?
+        try:
+            test_summary_report_index = lines.index("Test Summary Report")
 
-        if not test_result_lines:
-            return cmd.rc
+            del lines[0:test_summary_report_index + 2]
 
-        test_result_line = test_result_lines[0]
+            re_test_result = re.compile("^Result: (PASS|FAIL)$|Tests: \d+ Failed: (\d+)\)|Files=\d+, Tests=(\d+)")
 
-        success = test_result_line[0]
+            mos = map(lambda line: re_test_result.search(line), lines)
+            test_result_lines = [mo.groups() for mo in mos if mo]
 
-        if success:
-            failed = 0
+            for line in test_result_lines:
+                if line[0] == 'PASS':
+                    rc = SUCCESS
+                elif line[0] == 'FAIL':
+                    rc = FAILURE
+                elif line[1]:
+                    failed += int(line[1])
+                elif line[2]:
+                    total = int(line[2])
 
-            test_totals_line = test_result_lines[1]
-            total_str = test_totals_line[3]
+        except ValueError: # Nope, it's the old version
+            re_test_result = re.compile("^(All tests successful)|(\d+)/(\d+) subtests failed|Files=\d+, Tests=(\d+),")
 
-            rc = SUCCESS
-        else:
-            failed_str = test_result_line[1]
-            failed = int(failed_str)
+            mos = map(lambda line: re_test_result.search(line), lines)
+            test_result_lines = [mo.groups() for mo in mos if mo]
 
-            total_str = test_result_line[2]
+            if test_result_lines:
+                test_result_line = test_result_lines[0]
 
-            rc = FAILURE
+                success = test_result_line[0]
 
-        total = int(total_str)
-        passed = total - failed
+                if success:
+                    failed = 0
 
-        self.setTestResults(total=total, failed=failed, passed=passed)
+                    test_totals_line = test_result_lines[1]
+                    total_str = test_totals_line[3]
+                    
+                    rc = SUCCESS
+                else:
+                    failed_str = test_result_line[1]
+                    failed = int(failed_str)
+
+                    total_str = test_result_line[2]
+
+                    rc = FAILURE
+
+                total = int(total_str)
+
+        if total:
+            passed = total - failed
+
+            self.setTestResults(total=total, failed=failed, passed=passed)
 
         return rc
