@@ -9,9 +9,11 @@ from twisted.trial import unittest
 
 from buildbot import interfaces
 from buildbot.sourcestamp import SourceStamp
-from buildbot.process.base import BuildRequest
-from buildbot.status import builder, base, words
+from buildbot.process.base import BuildRequest, Build
+from buildbot.status import builder, base, words, progress
 from buildbot.changes.changes import Change
+from buildbot.process.builder import Builder
+from time import sleep
 
 mail = None
 try:
@@ -19,7 +21,7 @@ try:
 except ImportError:
     pass
 from buildbot.status import progress, client # NEEDS COVERAGE
-from buildbot.test.runutils import RunMixin
+from buildbot.test.runutils import RunMixin, setupBuildStepStatus
 
 class MyStep:
     build = None
@@ -95,6 +97,8 @@ class MyStatus:
         return self.url
     def getURLForThing(self, thing):
         return None
+    def getProjectName(self):
+        return "myproj"
 
 class MyBuilder(builder.BuilderStatus):
     nextBuildNumber = 0
@@ -123,6 +127,36 @@ class MyLookup:
             d.callback(user + "@" + "dev.com")
         return d
 
+def customTextMailMessage(attrs):
+    logLines = 3
+    text = list()
+    text.append("STATUS: %s" % attrs['result'].title())
+    text.append("")
+    text.extend([c.asText() for c in attrs['changes']])
+    text.append("")
+    name, url, lines = attrs['logs'][-1]
+    text.append("Last %d lines of '%s':" % (logLines, name))
+    text.extend(["\t%s\n" % line for line in lines[len(lines)-logLines:]])
+    text.append("")
+    text.append("-buildbot")
+    return ("\n".join(text), 'plain')
+
+def customHTMLMailMessage(attrs):
+    logLines = 3
+    text = list()
+    text.append("<h3>STATUS <a href='%s'>%s</a>:</h3>" % (attrs['buildURL'],
+                                                          attrs['result'].title()))
+    text.append("<h4>Recent Changes:</h4>")
+    text.extend([c.asHTML() for c in attrs['changes']])
+    name, url, lines = attrs['logs'][-1]
+    text.append("<h4>Last %d lines of '%s':</h4>" % (logLines, name))
+    text.append("<p>")
+    text.append("<br>".join([line for line in lines[len(lines)-logLines:]]))
+    text.append("</p>")
+    text.append("<br>")
+    text.append("<b>-<a href='%s'>buildbot</a></b>" % attrs['buildbotURL'])
+    return ("\n".join(text), 'html')
+
 class Mail(unittest.TestCase):
 
     def setUp(self):
@@ -165,8 +199,8 @@ class Mail(unittest.TestCase):
         self.failUnless(len(self.messages) == 1)
         m,r = self.messages.pop()
         t = m.as_string()
-        self.failUnlessIn("To: bob@dev.com, recip2@example.com, "
-                          "recip@example.com\n", t)
+        self.failUnlessIn("To: bob@dev.com\n", t)
+        self.failUnlessIn("CC: recip2@example.com, recip@example.com\n", t)
         self.failUnlessIn("From: buildbot@example.com\n", t)
         self.failUnlessIn("Subject: buildbot success in PROJECT on builder1\n", t)
         self.failUnlessIn("Date: ", t)
@@ -270,6 +304,80 @@ class Mail(unittest.TestCase):
         mailer3.buildFinished("builder2", b2, b2.results)
         self.failUnlessEqual(len(self.messages), 1)
 
+    def testCustomTextMessage(self):
+        basedir = "test_custom_text_mesg"
+        os.mkdir(basedir)
+        mailer = MyMailer(fromaddr="buildbot@example.com", mode="problem",
+                          extraRecipients=["recip@example.com",
+                                           "recip2@example.com"],
+                          lookup=MyLookup(),
+                          customMesg=customTextMailMessage)
+        mailer.parent = self
+        mailer.status = self
+        self.messages = []
+
+        b1 = self.makeBuild(4, builder.FAILURE)
+        b1.setText(["snarkleack", "polarization", "failed"])
+        b1.blamelist = ["dev3", "dev3", "dev3", "dev4",
+                        "Thomas_Walters"]
+        b1.source.changes = (Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 123),
+                             Change(who = 'author2', files = ['file2'], comments = 'comment2', revision = 456))
+        b1.testlogs = [MyLog(basedir, 'compile', "Compile log here\n"),
+                       MyLog(basedir, 'test', "Test log here\nTest 1 failed\nTest 2 failed\nTest 3 failed\nTest 4 failed\n")]
+
+        mailer.buildFinished("builder1", b1, b1.results)
+        m,r = self.messages.pop()
+        t = m.as_string()
+        #
+        # Uncomment to review custom message
+        #
+        #self.fail(t)
+        self.failUnlessIn("comment1", t)
+        self.failUnlessIn("comment2", t)
+        self.failUnlessIn("Test 4 failed", t)
+
+
+    def testCustomHTMLMessage(self):
+        basedir = "test_custom_HTML_mesg"
+        os.mkdir(basedir)
+        mailer = MyMailer(fromaddr="buildbot@example.com", mode="problem",
+                          extraRecipients=["recip@example.com",
+                                           "recip2@example.com"],
+                          lookup=MyLookup(),
+                          customMesg=customHTMLMailMessage)
+        mailer.parent = self
+        mailer.status = self
+        self.messages = []
+
+        b1 = self.makeBuild(4, builder.FAILURE)
+        b1.setText(["snarkleack", "polarization", "failed"])
+        b1.blamelist = ["dev3", "dev3", "dev3", "dev4",
+                        "Thomas_Walters"]
+        b1.source.changes = (Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 123),
+                             Change(who = 'author2', files = ['file2'], comments = 'comment2', revision = 456))
+        b1.testlogs = [MyLog(basedir, 'compile', "Compile log here\n"),
+                       MyLog(basedir, 'test', "Test log here\nTest 1 failed\nTest 2 failed\nTest 3 failed\nTest 4 failed\n")]
+
+        mailer.buildFinished("builder1", b1, b1.results)
+        m,r = self.messages.pop()
+        t = m.as_string()
+        #
+        # Uncomment to review custom message
+        #
+        #self.fail(t)
+        self.failUnlessIn("<h4>Last 3 lines of 'step.test':</h4>", t)
+        self.failUnlessIn("<p>Changed by: <b>author2</b><br />", t)
+        self.failUnlessIn("Test 3 failed", t)
+
+    def testShouldAttachLog(self):
+        mailer = mail.MailNotifier(fromaddr="buildbot@example.com", addLogs=True)
+        self.assertTrue(mailer._shouldAttachLog('anything'))
+        mailer = mail.MailNotifier(fromaddr="buildbot@example.com", addLogs=False)
+        self.assertFalse(mailer._shouldAttachLog('anything'))
+        mailer = mail.MailNotifier(fromaddr="buildbot@example.com", addLogs=['something'])
+        self.assertFalse(mailer._shouldAttachLog('anything'))
+        self.assertTrue(mailer._shouldAttachLog('something'))
+
     def testFailure(self):
         mailer = MyMailer(fromaddr="buildbot@example.com", mode="problem",
                           extraRecipients=["recip@example.com",
@@ -291,14 +399,14 @@ class Mail(unittest.TestCase):
         self.failUnless(len(self.messages) == 1)
         m,r = self.messages.pop()
         t = m.as_string()
-        self.failUnlessIn("To: dev3@dev.com, dev4@dev.com, "
-                          "recip2@example.com, recip@example.com\n", t)
+        self.failUnlessIn("To: dev3@dev.com, dev4@dev.com\n", t)
+        self.failUnlessIn("CC: recip2@example.com, recip@example.com\n", t)
         self.failUnlessIn("From: buildbot@example.com\n", t)
         self.failUnlessIn("Subject: buildbot failure in PROJECT on builder1\n", t)
         self.failUnlessIn("The Buildbot has detected a new failure", t)
         self.failUnlessIn("BUILD FAILED: snarkleack polarization failed\n", t)
-        self.failUnlessEqual(r, ["dev3@dev.com", "dev4@dev.com",
-                                 "recip2@example.com", "recip@example.com"])
+        self.failUnlessEqual(set(r), set(["dev3@dev.com", "dev4@dev.com",
+                                 "recip2@example.com", "recip@example.com"]))
 
     def testLogs(self):
         basedir = "test_status_logs"
@@ -353,7 +461,6 @@ class Mail(unittest.TestCase):
                              'test', "Test log here\nTest 4 failed\n"),
                    ]
 
-        print "sending mail to", dest
         d = mailer.buildFinished("builder1", b1, b1.results)
         # When this fires, the mail has been sent, but the SMTP connection is
         # still up (because smtp.sendmail relies upon the server to hang up).
@@ -427,7 +534,10 @@ class Log(unittest.TestCase):
         self.failUnlessEqual(len(list(l.getChunks())), 4)
 
         self.failUnless(l.hasContents())
-        os.unlink(l.getFilename())
+        try:
+            os.unlink(l.getFilename())
+        except OSError:
+            os.unlink(l.getFilename() + ".bz2")
         self.failIf(l.hasContents())
 
     def TODO_testDuplicate(self):
@@ -492,7 +602,7 @@ class Log(unittest.TestCase):
         self.failUnlessEqual(l.getText(), 160*"a")
 
     def testReadlines(self):
-        l = MyLog(self.basedir, "chunks")
+        l = MyLog(self.basedir, "chunks1")
         l.addHeader("HEADER\n") # should be ignored
         l.addStdout("Some text\n")
         l.addStdout("Some More Text\nAnd Some More\n")
@@ -518,7 +628,7 @@ class Log(unittest.TestCase):
 
 
     def testChunks(self):
-        l = MyLog(self.basedir, "chunks")
+        l = MyLog(self.basedir, "chunks2")
         c1 = l.getChunks()
         l.addHeader("HEADER\n")
         l.addStdout("Some text\n")
@@ -559,7 +669,10 @@ class Log(unittest.TestCase):
         # now doctor it to look like a 0.6.4-era non-upgraded logfile
         l.entries = list(l.getChunks())
         del l.filename
-        os.unlink(l.getFilename())
+        try:
+            os.unlink(l.getFilename() + ".bz2")
+        except OSError:
+            os.unlink(l.getFilename())
         # now make sure we can upgrade it
         l.upgrade("upgrade")
         self.failUnlessEqual(l.getText(),
@@ -728,6 +841,35 @@ class Log(unittest.TestCase):
         return d
     testLargeSummary.timeout = 5
 
+
+class CompressLog(unittest.TestCase):
+    def testCompressLogs(self):
+        bss = setupBuildStepStatus("test-compress")
+        bss.build.builder.setLogCompressionLimit(1024)
+        l = bss.addLog('not-compress')
+        l.addStdout('a' * 512)
+        l.finish()
+        lc = bss.addLog('to-compress')
+        lc.addStdout('b' * 1024)
+        lc.finish()
+        d = bss.stepFinished(builder.SUCCESS)
+        self.failUnless(d is not None)
+        d.addCallback(self._verifyCompression, bss)
+        return d
+
+    def _verifyCompression(self, result, bss):
+        self.failUnless(len(bss.getLogs()), 2)
+        (ncl, cl) = bss.getLogs() # not compressed, compressed log
+        self.failUnless(os.path.isfile(ncl.getFilename()))
+        self.failIf(os.path.isfile(ncl.getFilename() + ".bz2"))
+        self.failIf(os.path.isfile(cl.getFilename()))
+        self.failUnless(os.path.isfile(cl.getFilename() + ".bz2"))
+        content = ncl.getText()
+        self.failUnless(len(content), 512)
+        content = cl.getText()
+        self.failUnless(len(content), 1024)
+        pass
+
 config_base = """
 from buildbot.process import factory
 from buildbot.steps import dummy
@@ -794,6 +936,10 @@ class STarget(base.StatusReceiver):
             print "EXP", step.getExpectations()
         if "step" in self.mode:
             return self
+    def stepTextChanged(self, build, step, text):
+        self.events.append(("stepTextChanged", step, text))
+    def stepText2Changed(self, build, step, text2):
+        self.events.append(("stepText2Changed", step, text2))
     def stepETAUpdate(self, build, step, ETA, expectations):
         self.events.append(("stepETAUpdate", build, step, ETA, expectations))
         self.announce()
@@ -874,7 +1020,7 @@ class Subscription(RunMixin, unittest.TestCase):
         t1.events = []
 
         c = interfaces.IControl(self.master)
-        req = BuildRequest("forced build for testing", SourceStamp())
+        req = BuildRequest("forced build for testing", SourceStamp(), 'test_builder')
         c.getBuilder("dummy").requestBuild(req)
         d = req.waitUntilFinished()
         d2 = self.master.botmaster.waitUntilBuilderIdle("dummy")
@@ -903,9 +1049,12 @@ class Subscription(RunMixin, unittest.TestCase):
                               "builderChangedState", # idle
                               "builderChangedState", # building
                               "buildStarted",
-                              "stepStarted", "stepETAUpdate", "stepFinished",
+                              "stepStarted", "stepETAUpdate", 
+                              "stepTextChanged", "stepFinished",
                               "stepStarted", "stepETAUpdate",
-                              "logStarted", "logFinished", "stepFinished",
+                              "stepTextChanged", "logStarted", "logFinished",
+                              "stepTextChanged", "stepText2Changed",
+                              "stepFinished",
                               "buildFinished",
                               "builderChangedState", # idle
                               ])
@@ -922,7 +1071,6 @@ class Subscription(RunMixin, unittest.TestCase):
         self.failUnlessEqual(b.getResponsibleUsers(), [])
         self.failUnless(b.isFinished())
         self.failUnlessEqual(b.getText(), ['build', 'successful'])
-        self.failUnlessEqual(b.getColor(), "green")
         self.failUnlessEqual(b.getResults(), builder.SUCCESS)
 
         steps = b.getSteps()
@@ -959,7 +1107,7 @@ class Subscription(RunMixin, unittest.TestCase):
         self.t4 = t4 = STarget(["builder", "build", "eta"])
         self.master.getStatus().subscribe(t4)
         c = interfaces.IControl(self.master)
-        req = BuildRequest("forced build for testing", SourceStamp())
+        req = BuildRequest("forced build for testing", SourceStamp(), 'test_builder')
         c.getBuilder("dummy").requestBuild(req)
         d = req.waitUntilFinished()
         d2 = self.master.botmaster.waitUntilBuilderIdle("dummy")
@@ -1015,8 +1163,8 @@ class ContactTester(unittest.TestCase):
         self.failUnlessEqual(irc.message, "The following events are being notified: ['finished']", "off started")
 
         irc.message = ""
-        irc.command_NOTIFY("on success failed exception", "mynick")
-        self.failUnlessEqual(irc.message, "The following events are being notified: ['failed', 'finished', 'exception', 'success']", "on multiple events")
+        irc.command_NOTIFY("on success failure exception", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: ['failure', 'finished', 'exception', 'success']", "on multiple events")
 
     def test_notification_default(self):
         irc = MyContact()
@@ -1107,7 +1255,7 @@ class ContactTester(unittest.TestCase):
             Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
             )
 
-        irc.command_NOTIFY("on failed", "mynick")
+        irc.command_NOTIFY("on failure", "mynick")
 
         irc.message = ""
         irc.buildStarted(my_builder.getName(), my_build)
@@ -1156,12 +1304,211 @@ class ContactTester(unittest.TestCase):
         irc.buildFinished(my_builder.getName(), my_build, None)
         self.failUnlessEqual(irc.message, "", "No finish notification generated on exception with notify_events=['exception']")
 
+    def do_x_to_y_notification_test(self, notify, previous_result, new_result, expected_msg):
+        irc = MyContact()
+        irc.command_NOTIFY("on %s" % notify, "mynick")
+
+        my_builder = MyBuilder("builder834")
+        my_build = MyIrcBuild(my_builder, 862, builder.FAILURE)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
+            )
+
+        previous_build = MyIrcBuild(my_builder, 861, previous_result)
+        my_build.setPreviousBuild(previous_build)
+
+        irc.message = ""
+        my_build.results = new_result
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, expected_msg, "Finish notification generated on failure with notify_events=['successToFailure']")
+
+    def test_notification_successToFailure(self):
+        self.do_x_to_y_notification_test(notify="successToFailure", previous_result=builder.SUCCESS, new_result=builder.FAILURE,
+                                         expected_msg="build #862 of builder834 is complete: Failure [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="successToFailure", previous_result=builder.SUCCESS, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="successToFailure", previous_result=builder.SUCCESS, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="successToFailure", previous_result=builder.SUCCESS, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_successToWarnings(self):
+        self.do_x_to_y_notification_test(notify="successToWarnings", previous_result=builder.SUCCESS, new_result=builder.WARNINGS,
+                                         expected_msg="build #862 of builder834 is complete: Warnings [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="successToWarnings", previous_result=builder.SUCCESS, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="successToWarnings", previous_result=builder.SUCCESS, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="successToWarnings", previous_result=builder.SUCCESS, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_successToException(self):
+        self.do_x_to_y_notification_test(notify="successToException", previous_result=builder.SUCCESS, new_result=builder.EXCEPTION,
+                                         expected_msg="build #862 of builder834 is complete: Exception [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="successToException", previous_result=builder.SUCCESS, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="successToException", previous_result=builder.SUCCESS, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="successToException", previous_result=builder.SUCCESS, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+
+
+
+
+    def test_notification_failureToSuccess(self):
+        self.do_x_to_y_notification_test(notify="failureToSuccess", previous_result=builder.FAILURE,new_result=builder.SUCCESS,
+                                         expected_msg="build #862 of builder834 is complete: Success [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="failureToSuccess", previous_result=builder.FAILURE,new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="failureToSuccess", previous_result=builder.FAILURE,new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="failureToSuccess", previous_result=builder.FAILURE,new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_failureToWarnings(self):
+        self.do_x_to_y_notification_test(notify="failureToWarnings", previous_result=builder.FAILURE, new_result=builder.WARNINGS,
+                                         expected_msg="build #862 of builder834 is complete: Warnings [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="failureToWarnings", previous_result=builder.FAILURE, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="failureToWarnings", previous_result=builder.FAILURE, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="failureToWarnings", previous_result=builder.FAILURE, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_failureToException(self):
+        self.do_x_to_y_notification_test(notify="failureToException", previous_result=builder.FAILURE, new_result=builder.EXCEPTION,
+                                         expected_msg="build #862 of builder834 is complete: Exception [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="failureToException", previous_result=builder.FAILURE, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="failureToException", previous_result=builder.FAILURE, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="failureToException", previous_result=builder.FAILURE, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+
+
+
+
+    def test_notification_warningsToFailure(self):
+        self.do_x_to_y_notification_test(notify="warningsToFailure", previous_result=builder.WARNINGS, new_result=builder.FAILURE,
+                                         expected_msg="build #862 of builder834 is complete: Failure [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="warningsToFailure", previous_result=builder.WARNINGS, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="warningsToFailure", previous_result=builder.WARNINGS, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="warningsToFailure", previous_result=builder.WARNINGS, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_warningsToSuccess(self):
+        self.do_x_to_y_notification_test(notify="warningsToSuccess", previous_result=builder.WARNINGS, new_result=builder.SUCCESS,
+                                         expected_msg="build #862 of builder834 is complete: Success [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="warningsToSuccess", previous_result=builder.WARNINGS, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="warningsToSuccess", previous_result=builder.WARNINGS, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="warningsToSuccess", previous_result=builder.WARNINGS, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_warningsToException(self):
+        self.do_x_to_y_notification_test(notify="warningsToException", previous_result=builder.WARNINGS, new_result=builder.EXCEPTION,
+                                         expected_msg="build #862 of builder834 is complete: Exception [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="warningsToException", previous_result=builder.WARNINGS, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="warningsToException", previous_result=builder.WARNINGS, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="warningsToException", previous_result=builder.WARNINGS, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+
+
+
+    def test_notification_exceptionToFailure(self):
+        self.do_x_to_y_notification_test(notify="exceptionToFailure", previous_result=builder.EXCEPTION, new_result=builder.FAILURE,
+                                         expected_msg="build #862 of builder834 is complete: Failure [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToFailure", previous_result=builder.EXCEPTION, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToFailure", previous_result=builder.EXCEPTION, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToFailure", previous_result=builder.EXCEPTION, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_exceptionToWarnings(self):
+        self.do_x_to_y_notification_test(notify="exceptionToWarnings", previous_result=builder.EXCEPTION, new_result=builder.WARNINGS,
+                                         expected_msg="build #862 of builder834 is complete: Warnings [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToWarnings", previous_result=builder.EXCEPTION, new_result=builder.SUCCESS,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToWarnings", previous_result=builder.EXCEPTION, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToWarnings", previous_result=builder.EXCEPTION, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+    def test_notification_exceptionToSuccess(self):
+        self.do_x_to_y_notification_test(notify="exceptionToSuccess", previous_result=builder.EXCEPTION, new_result=builder.SUCCESS,
+                                         expected_msg="build #862 of builder834 is complete: Success [step1 step2]  Build details are at http://myserver/mypath?build=765" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToSuccess", previous_result=builder.EXCEPTION, new_result=builder.EXCEPTION,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToSuccess", previous_result=builder.EXCEPTION, new_result=builder.FAILURE,
+                                         expected_msg = "" )
+
+        self.do_x_to_y_notification_test(notify="exceptionToSuccess", previous_result=builder.EXCEPTION, new_result=builder.WARNINGS,
+                                         expected_msg = "" )
+
+    def test_notification_set_in_config(self):
+        irc = MyContact(channel = MyChannel(notify_events = {'success': 1}))
+
+        my_builder = MyBuilder("builder834")
+        my_build = MyIrcBuild(my_builder, 862, builder.SUCCESS)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
+            )
+
+        irc.message = ""
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "build #862 of builder834 is complete: Success [step1 step2]  Build details are at http://myserver/mypath?build=765", "Finish notification generated on success with notify_events=['success']")
+
 class MyIrcBuild(builder.BuildStatus):
     results = None
 
     def __init__(self, parent, number, results):
         builder.BuildStatus.__init__(self, parent, number)
         self.results = results
+        self.previousBuild = None
 
     def getResults(self):
         return self.results
@@ -1169,6 +1516,12 @@ class MyIrcBuild(builder.BuildStatus):
     def getText(self):
         return ('step1', 'step2')
 
+    def setPreviousBuild(self, pb):
+        self.previousBuild = pb
+
+    def getPreviousBuild(self):
+        return self.previousBuild
+
 class URLProducer:
     def getURLForThing(self, build):
         return 'http://myserver/mypath?build=765'
@@ -1176,9 +1529,10 @@ class URLProducer:
 class MyChannel:
     categories = None
     status = URLProducer()
+    notify_events = {}
 
-    def __init__(self):
-        pass
+    def __init__(self, notify_events = {}):
+        self.notify_events = notify_events
 
 class MyContact(words.Contact):
     message = ""
@@ -1228,3 +1582,50 @@ class StepStatistics(unittest.TestCase):
         self.failUnlessEqual(
             status.getSummaryStatistic('test-prop', operator.add, 13), 20,
             'Sum property across the build with initial value')
+
+class BuildExpectation(unittest.TestCase):
+    class MyBuilderStatus:
+        implements(interfaces.IBuilderStatus)
+
+        def setSlavenames(self, slaveName):
+            pass
+
+    class MyBuilder(Builder):
+        def __init__(self, name):
+            Builder.__init__(self, {
+                    'name': name,
+                    'builddir': '/tmp/somewhere',
+                    'factory': 'aFactory'
+                    }, BuildExpectation.MyBuilderStatus())
+
+    class MyBuild(Build):
+        def __init__(self, b):
+            self.builder = b
+            self.remote = None
+
+            step1_progress = progress.StepProgress('step1', ['elapsed'])
+            self.progress = progress.BuildProgress([step1_progress])
+            step1_progress.setBuildProgress(self.progress)
+
+            step1_progress.start()
+            sleep(1);
+            step1_progress.finish()
+
+            self.deferred = defer.Deferred()
+            self.locks = []
+            self.build_status = builder.BuildStatus(b.builder_status, 1)
+
+
+    def testBuildExpectation_BuildSuccess(self):
+        b = BuildExpectation.MyBuilder("builder1")
+        build = BuildExpectation.MyBuild(b)
+
+        build.buildFinished(['sometext'], builder.SUCCESS)
+        self.failIfEqual(b.expectations.expectedBuildTime(), 0, 'Non-Zero expectation for a failed build')
+
+    def testBuildExpectation_BuildFailure(self):
+        b = BuildExpectation.MyBuilder("builder1")
+        build = BuildExpectation.MyBuild(b)
+
+        build.buildFinished(['sometext'], builder.FAILURE)
+        self.failUnlessEqual(b.expectations, None, 'Zero expectation for a failed build')
